<?php

/**
 * @file
 * Handles request to AvaTax Calc for sales tax values.
 */

/**
 * Makes a request to Avatax Calc for sales tax values.
 *
 * @param object $order
 *   The current order object.
 *
 * @param array $ava_args
 *   An array containing order delivery details.
 *
 * @return bool|array
 *   FALSE if the tax calculation failed.
 *   An array containing tax amount, taxable amount, total order amount.
 */
function uc_tax_avatax_get_tax($order, $ava_args) {

  // Include in all Avalara Scripts.
  require_once 'ava_tax.php';
  require_once 'credentials.php';

  // Avatax Get Tax rates.
  // Create new origin, destination, client and tax request objects.
  $client = new TaxServiceSoap($ava_args['use_mode']);
  $request = new GetTaxRequest();
  $origin = new Address();
  $destination = new Address();

  // Populate the From address.
  $origin->setLine1($ava_args['primary_street1']);
  $origin->setLine2($ava_args['primary_street2']);
  $origin->setCity($ava_args['primary_city']);
  $origin->setRegion($ava_args['primary_state']);
  $origin->setPostalCode($ava_args['primary_zip']);

  // Populate the Destination address.
  $destination->setLine1($ava_args['street1']);
  $destination->setLine2($ava_args['street2']);
  $destination->setCity($ava_args['city']);
  $destination->setRegion($ava_args['state']);
  $destination->setCountry($ava_args['country']);
  $destination->setPostalCode($ava_args['zip']);

  // Set request object.
  $request->setOriginAddress($origin);
  $request->setDestinationAddress($destination);
  $request->setcompanycode($ava_args['company_code']);
  $request->setDetailLevel(DetailLevel::$Tax);

  if ($order->order_id == 0) {
    $request->setDocType('SalesOrder');
  }
  else {
    $request->setDocType('SalesInvoice');
  }

  $date_time = new DateTime();

  $request->setDocCode('uc-' . $order->order_id . '');

  $request->setDocDate(date_format($date_time, "Y-m-d"));
  $request->setSalespersonCode('Ubercart');
  $request->setCustomerCode($ava_args['user_id']);
  $request->setLocationCode('');
  $request->setCustomerUsageType('');

  $i = 1;
  foreach ($order->products as $prod) {
    ${'line' . $i} = new Line();
    ${'line' . $i}->setNo($i);
    ${'line' . $i}->setItemCode($prod->model);
    ${'line' . $i}->setDescription($prod->title);
    ${'line' . $i}->setTaxCode('');
    ${'line' . $i}->setQty($prod->qty);
    ${'line' . $i}->setAmount($prod->qty * $prod->price);
    ${'line' . $i}->setDiscounted('false');
    ${'line' . $i}->setRevAcct('');
    ${'line' . $i}->setRef1('');
    ${'line' . $i}->setRef2('');
    ${'line' . $i}->setCustomerUsageType('');
    $lines[] = ${'line' . $i};
    $i++;
  }
  foreach ($order->line_items as $key => $item) {
    if (in_array($item['type'], array('coupon'))) {
      ${'line' . $i} = new Line();
      ${'line' . $i}->setNo($i);
      ${'line' . $i}->setItemCode($item['type']);
      ${'line' . $i}->setDescription($item['title']);
      ${'line' . $i}->setTaxCode('');
      ${'line' . $i}->setQty(1);
      ${'line' . $i}->setAmount($item['amount']);
      ${'line' . $i}->setDiscounted('false');
      ${'line' . $i}->setRevAcct('');
      ${'line' . $i}->setRef1('');
      ${'line' . $i}->setRef2('');
      ${'line' . $i}->setCustomerUsageType('');
      $lines[] = ${'line' . $i};
      $i++;
    }
    elseif (in_array($item['type'], array('shipping'))) {
      ${'line' . $i} = new Line();
      ${'line' . $i}->setNo($i);
      ${'line' . $i}->setItemCode($item['type']);
      ${'line' . $i}->setDescription($item['title']);
      ${'line' . $i}->setTaxCode($ava_args['shipcode']);
      ${'line' . $i}->setQty(1);
      ${'line' . $i}->setAmount($item['amount']);
      ${'line' . $i}->setDiscounted('false');
      ${'line' . $i}->setRevAcct('');
      ${'line' . $i}->setRef1('');
      ${'line' . $i}->setRef2('');
      ${'line' . $i}->setCustomerUsageType('');
      $lines[] = ${'line' . $i};
      $i++;
    }
    elseif (in_array($item['type'], array('generic'))) {
      ${'line' . $i} = new Line();
      ${'line' . $i}->setNo($i);
      ${'line' . $i}->setItemCode($item['type']);
      ${'line' . $i}->setDescription($item['title']);
      ${'line' . $i}->setTaxCode('');
      ${'line' . $i}->setQty(1);
      ${'line' . $i}->setAmount($item['amount']);
      ${'line' . $i}->setDiscounted('false');
      ${'line' . $i}->setRevAcct('');
      ${'line' . $i}->setRef1('');
      ${'line' . $i}->setRef2('');
      ${'line' . $i}->setCustomerUsageType('');
      $lines[] = ${'line' . $i};
      $i++;
    }
  }

  $request->setLines($lines);

  // Try AvaTax.
  try {
    $get_tax_result = $client->getTax($request);
    if ($get_tax_result->getResultCode() == SeverityLevel::$Success) {
      $tax_data = array(
        'tax_amount' => $get_tax_result->getTotalTax(),
        'taxable_amount' => $get_tax_result->getTotalTaxable(),
        'total_amount' => $get_tax_result->getTotalAmount(),
      );
    }
    else {
      foreach ($get_tax_result->getMessages() as $msg) {
        drupal_set_message(filter_xss('AvaTax error: ' . $msg->getName() . ": " . check_plain($msg->getSummary()) . '.'), 'error');
      }
      return FALSE;
    }
  }

  catch (SoapFault $exception) {
    $msg = 'SOAP Exception: ';
    if ($exception) {
      $msg .= $exception->faultstring;
    }
    drupal_set_message(filter_xss('AvaTax message is: ' . $msg . '.'));
    drupal_set_message(filter_xss('AvaTax last request is: ' . $client->__getLastRequest() . '.'));
    drupal_set_message(filter_xss('AvaTax last response is: ' . $client->__getLastResponse() . '.'));
    return FALSE;
  }
  return $tax_data;
}
