<?php

/**
 * @file
 * Handles request to AvaTax Calc for sales tax values.
 */

/**
 * Gets the tax amount for the order based on the delivery address.
 *
 * @param object $order
 *   The current order object.
 *
 * @param array $ava_args
 *   An array containing from & to delivery details.
 *
 * @return bool|array
 *   FALSE if the tax calculation failed.
 *   An array containing tax amount, taxable amount, total order amount.
 */
function uc_tax_avatax_get_tax($order, $ava_args) {

  // Include in all Avalara Scripts.
  require_once 'ava_tax.php';
  require_once 'credentials.php';

  // Avatax Get Tax rates.
  // Create new origin, destination, client and tax request objects.
  $client = new TaxServiceSoap($ava_args['use_mode']);
  $request = new GetTaxRequest();
  $origin = new Address();
  $destination = new Address();

  // Populate the From address.
  $origin->setLine1($ava_args['primary_street1']);
  $origin->setLine2($ava_args['primary_street2']);
  $origin->setCity($ava_args['primary_city']);
  $origin->setRegion($ava_args['primary_state']);
  $origin->setPostalCode($ava_args['primary_zip']);

  // Populate the Destination address.
  $destination->setLine1($ava_args['street1']);
  $destination->setLine2($ava_args['street2']);
  $destination->setCity($ava_args['city']);
  $destination->setRegion($ava_args['state']);
  $destination->setCountry($ava_args['country']);
  $destination->setPostalCode($ava_args['zip']);

  // Set request object.
  $request->setOriginAddress($origin);
  $request->setDestinationAddress($destination);
  $request->setcompanycode($ava_args['company_code']);
  $request->setDetailLevel(DetailLevel::$Tax);

  if ($order->order_id == 0) {
    $request->setDocType('SalesOrder');
  }
  else {
    $request->setDocType('SalesInvoice');
  }

  $date_time = new DateTime();

  $request->setDocCode('uc-' . $order->order_id . '');

  $request->setDocDate(date_format($date_time, "Y-m-d"));
  $request->setSalespersonCode('Ubercart');
  $request->setCustomerCode($ava_args['user_id']);
  $request->setLocationCode('');
  $request->setCustomerUsageType('');

  // Required: Change 'not_set' to one of the following modes.
  // Valid choices are 'compute' 'item_code' 'tax_code' 'taxonomy_tax_code'
  $pro_mode = 'not_set';

  // Return error if AvaTax Pro has been selected without above configured.
  if ($pro_mode == 'not_set') {
    drupal_set_message(filter_xss('Ubercart Connector for AvaTax error. Selection of Avatax Pro requires (line 74) $pro_mode to be configured.'), 'error');
    $tax_data = array();
    return $tax_data;
  }

  $i = 1;

  // To Compute Sales Tax codes - Sales Tax Code based on skew being matched.
  if ($pro_mode == 'compute') {
    foreach ($order->products as $prod) {
      // If the skew ($prod->model) matches a value in the array.
      // The line item will not be taxed for tax code - Not Taxable NT.
      if (in_array($prod->model, array('NT-Skew1', 'NT-Skew2'))) {
        // Non Taxable Product.
        $tax_code = 'NT';
        ${'line' . $i} = new Line();
        ${'line' . $i}->setNo($i);
        ${'line' . $i}->setItemCode($prod->model);
        ${'line' . $i}->setDescription($prod->title);
        ${'line' . $i}->setTaxCode($tax_code);
        ${'line' . $i}->setQty($prod->qty);
        ${'line' . $i}->setAmount($prod->qty * $prod->price);
        ${'line' . $i}->setDiscounted('false');
        ${'line' . $i}->setRevAcct('');
        ${'line' . $i}->setRef1('');
        ${'line' . $i}->setRef2('');
        ${'line' . $i}->setCustomerUsageType('');
        $lines[] = ${'line' . $i};
        $i++;
      }
      // If the skew ($prod->model) matches a value in the array.
      // The line item will be taxed for tax code - value of $tax_code.
      elseif (in_array($prod->model, array('CUST-Skew1', 'CUST-Skew2'))) {
        // Edit as required - a custom tax code or an AvaTax tax code.
        $tax_code = 'CUST0100';
        ${'line' . $i} = new Line();
        ${'line' . $i}->setNo($i);
        ${'line' . $i}->setItemCode($prod->model);
        ${'line' . $i}->setDescription($prod->title);
        ${'line' . $i}->setTaxCode($tax_code);
        ${'line' . $i}->setQty($prod->qty);
        ${'line' . $i}->setAmount($prod->qty * $prod->price);
        ${'line' . $i}->setDiscounted('false');
        ${'line' . $i}->setRevAcct('');
        ${'line' . $i}->setRef1('');
        ${'line' . $i}->setRef2('');
        ${'line' . $i}->setCustomerUsageType('');
        $lines[] = ${'line' . $i};
        $i++;
      }
      // If no earlier matches - the line item taxed at value of $tax_code.
      else {
        // Edit as required - a custom tax code or an AvaTax tax code.
        $tax_code = 'P0000000';
        ${'line' . $i} = new Line();
        ${'line' . $i}->setNo($i);
        ${'line' . $i}->setItemCode($prod->model);
        ${'line' . $i}->setDescription($prod->title);
        ${'line' . $i}->setTaxCode($tax_code);
        ${'line' . $i}->setQty($prod->qty);
        ${'line' . $i}->setAmount($prod->qty * $prod->price);
        ${'line' . $i}->setDiscounted('false');
        ${'line' . $i}->setRevAcct('');
        ${'line' . $i}->setRef1('');
        ${'line' . $i}->setRef2('');
        ${'line' . $i}->setCustomerUsageType('');
        $lines[] = ${'line' . $i};
        $i++;
      }
    }
  }

  // To assign an Item Code from a custom field in the product node.
  if ($pro_mode == 'item_code') {

    // Enter the CCK field name for the tax code data - do not include field_.
    $tax_field_name = 'not_set';
    // Return error if no $tax_field_name configured.
    if ($tax_field_name == 'not_set') {
      drupal_set_message(filter_xss('Ubercart Connector for AvaTax error. Selection of Avatax Pro in "item_code" mode requires (line 154) $tax_field_name to be configured.'), 'error');
      $tax_data = array();
      return $tax_data;
    }

    // Enter the product content type - Ubercart default is product.
    $product_content_type = 'not_set';

    // Return error if no $product_content_type above configured.
    if ($product_content_type == 'not_set') {
      drupal_set_message(filter_xss('Ubercart Connector for AvaTax error. Selection of Avatax Pro in "item_code" mode requires (line 163) $product_content_type to be configured.'), 'error');
      $tax_data = array();
      return $tax_data;
    }

    foreach ($order->products as $prod) {
      $item_code = $prod->{'field_' . $tax_field_name}[0]['value'];

      // Look up CCK field if order form has not fully populated the line item.
      if ($item_code == '') {
        $prod_node->nid = $prod->nid;
        $prod_node->vid = $prod->vid;
        $prod_node->type = $product_content_type;
        $cck_fields = content_storage('load', $prod_node);
        $item_code = $cck_fields['field_' . $tax_field_name][0]['value'];

        // If no tax code is found for the CCK field - use product skew.
        if ($item_code == '') {
          $item_code = $prod->model;
        }
      }

      ${'line' . $i} = new Line();
      ${'line' . $i}->setNo($i);
      ${'line' . $i}->setItemCode($item_code);
      ${'line' . $i}->setDescription($prod->title);
      ${'line' . $i}->setTaxCode('');
      ${'line' . $i}->setQty($prod->qty);
      ${'line' . $i}->setAmount($prod->qty * $prod->price);
      ${'line' . $i}->setDiscounted('false');
      ${'line' . $i}->setRevAcct('');
      ${'line' . $i}->setRef1('');
      ${'line' . $i}->setRef2('');
      ${'line' . $i}->setCustomerUsageType('');
      $lines[] = ${'line' . $i};
      $i++;
    }
  }

  // To assign an AvaTax Sales Tax Code from a custom field of the product node.
  if ($pro_mode == 'tax_code') {

    // Enter the CCK field name for the tax code data - do not include field_.
    $tax_field_name = 'not_set';
    // Return error if no $tax_field_name configured.
    if ($tax_field_name == 'not_set') {
      drupal_set_message(filter_xss('Ubercart Connector for AvaTax error. Selection of Avatax Pro in "item_code" mode requires (line 210) $tax_field_name to be configured.'), 'error');
      $tax_data = array();
      return $tax_data;
    }

    // Enter the product content type - Ubercart default is product.
    $product_content_type = 'not_set';

    // Return error if no $product_content_type above configured.
    if ($product_content_type == 'not_set') {
      drupal_set_message(filter_xss('Ubercart Connector for AvaTax error. Selection of Avatax Pro in "item_code" mode requires (line 222) $product_content_type to be configured.'), 'error');
      $tax_data = array();
      return $tax_data;
    }

    foreach ($order->products as $prod) {
      $tax_code = $prod->{'field_' . $tax_field_name}[0]['value'];

      // Look up CCK field if order form has not fully populated the line item.
      if ($tax_code == '') {
        $prod_node->nid = $prod->nid;
        $prod_node->vid = $prod->vid;
        $prod_node->type = $product_content_type;
        $cck_fields = content_storage('load', $prod_node);
        $tax_code = $cck_fields['field_' . $tax_field_name][0]['value'];
      }

      ${'line' . $i} = new Line();
      ${'line' . $i}->setNo($i);
      ${'line' . $i}->setItemCode($prod->model);
      ${'line' . $i}->setDescription($prod->title);
      ${'line' . $i}->setTaxCode($tax_code);
      ${'line' . $i}->setQty($prod->qty);
      ${'line' . $i}->setAmount($prod->qty * $prod->price);
      ${'line' . $i}->setDiscounted('false');
      ${'line' . $i}->setRevAcct('');
      ${'line' . $i}->setRef1('');
      ${'line' . $i}->setRef2('');
      ${'line' . $i}->setCustomerUsageType('');
      $lines[] = ${'line' . $i};
      $i++;
    }
  }

  // To assign a Tax Code from a taxonomy field in the product node.
  if ($pro_mode == 'taxonomy_tax_code') {
    // Enter the taxonomy vid as a string value.
    $tax_field_vid = 'not_set';
    // Return error if no $tax_field_vid configured.
    if ($tax_field_vid == 'not_set') {
      drupal_set_message(filter_xss('Ubercart Connector for AvaTax error. Selection of Avatax Pro in "taxonomy_tax_code" mode requires (line 260) $tax_field_vid to be configured.'), 'error');
      $tax_data = array();
      return $tax_data;
    }

    foreach ($order->products as $prod) {
      if ($prod->taxonomy) {
        $tax_code = '';
        foreach ($prod->taxonomy as $tid) {
          if ($tid->vid == $tax_field_vid) {
            $tax_code = $tid->name;
          }
        }
      }

      // Look up taxonomy term if order form has not populated the line item.
      if (!$prod->taxonomy) {
        $tax_code = '';
        $prod_raw = node_load($prod->nid, NULL, TRUE);
        if ($prod_raw->taxonomy) {
          foreach ($prod_raw->taxonomy as $tid) {
            if ($tid->vid == $tax_field_vid) {
              $tax_code = $tid->name;
            }
          }
        }
      }

      ${'line' . $i} = new Line();
      ${'line' . $i}->setNo($i);
      ${'line' . $i}->setItemCode($prod->model);
      ${'line' . $i}->setDescription($prod->title);
      ${'line' . $i}->setTaxCode($tax_code);
      ${'line' . $i}->setQty($prod->qty);
      ${'line' . $i}->setAmount($prod->qty * $prod->price);
      ${'line' . $i}->setDiscounted('false');
      ${'line' . $i}->setRevAcct('');
      ${'line' . $i}->setRef1('');
      ${'line' . $i}->setRef2('');
      ${'line' . $i}->setCustomerUsageType('');
      $lines[] = ${'line' . $i};
      $i++;
    }
  }

  foreach ($order->line_items as $key => $item) {
    if (in_array($item['type'], array('coupon'))) {
      ${'line' . $i} = new Line();
      ${'line' . $i}->setNo($i);
      ${'line' . $i}->setItemCode($item['type']);
      ${'line' . $i}->setDescription($item['title']);
      ${'line' . $i}->setTaxCode('');
      ${'line' . $i}->setQty(1);
      ${'line' . $i}->setAmount($item['amount']);
      ${'line' . $i}->setDiscounted('false');
      ${'line' . $i}->setRevAcct('');
      ${'line' . $i}->setRef1('');
      ${'line' . $i}->setRef2('');
      ${'line' . $i}->setCustomerUsageType('');
      $lines[] = ${'line' . $i};
      $i++;
    }
    elseif (in_array($item['type'], array('shipping'))) {
      ${'line' . $i} = new Line();
      ${'line' . $i}->setNo($i);
      ${'line' . $i}->setItemCode($item['type']);
      ${'line' . $i}->setDescription($item['title']);
      ${'line' . $i}->setTaxCode($ava_args['shipcode']);
      ${'line' . $i}->setQty(1);
      ${'line' . $i}->setAmount($item['amount']);
      ${'line' . $i}->setDiscounted('false');
      ${'line' . $i}->setRevAcct('');
      ${'line' . $i}->setRef1('');
      ${'line' . $i}->setRef2('');
      ${'line' . $i}->setCustomerUsageType('');
      $lines[] = ${'line' . $i};
      $i++;
    }
    elseif (in_array($item['type'], array('generic'))) {
      ${'line' . $i} = new Line();
      ${'line' . $i}->setNo($i);
      ${'line' . $i}->setItemCode($item['type']);
      ${'line' . $i}->setDescription($item['title']);
      ${'line' . $i}->setTaxCode('');
      ${'line' . $i}->setQty(1);
      ${'line' . $i}->setAmount($item['amount']);
      ${'line' . $i}->setDiscounted('false');
      ${'line' . $i}->setRevAcct('');
      ${'line' . $i}->setRef1('');
      ${'line' . $i}->setRef2('');
      ${'line' . $i}->setCustomerUsageType('');
      $lines[] = ${'line' . $i};
      $i++;
    }
  }

  $request->setLines($lines);

  // Try AvaTax.
  try {
    $get_tax_result = $client->getTax($request);
    if ($get_tax_result->getResultCode() == SeverityLevel::$Success) {
      $tax_data = array(
        'tax_amount' => $get_tax_result->getTotalTax(),
        'taxable_amount' => $get_tax_result->getTotalTaxable(),
        'total_amount' => $get_tax_result->getTotalAmount(),
      );
    }
    else {
      foreach ($get_tax_result->getMessages() as $msg) {
        drupal_set_message(filter_xss('AvaTax error: ' . $msg->getName() . ": " . $msg->getSummary() . ''), 'error');
      }
      return FALSE;
    }
  }

  catch (SoapFault $exception) {
    $msg = 'SOAP Exception: ';
    if ($exception) {
      $msg .= $exception->faultstring;
    }
    drupal_set_message(filter_xss('AvaTax message is: ' . $msg . '.'));
    drupal_set_message(filter_xss('AvaTax last request is: ' . $client->__getLastRequest() . '.'));
    drupal_set_message(filter_xss('AvaTax last response is: ' . $client->__getLastResponse() . '.'));
    return FALSE;
  }
  return $tax_data;
}
